//////////////////////////////////////////////////////////////////////////////////////////////////////
//																									//
//	Cdigo fuente que implementa un cliente TCP/IP en el sistema BOLT, la comunicacin se establece	//
//	mediante el modulo QUECTEL UC15, que es un dispositivos que permite la conexin a travs la red //
//	de datos de telefona celular, en particular se utiliza el proveedor Telcel(se pueden usar otros//
//	proveedores de servicios de datos). Se utiliza la siguiente informacin para poder acceder a la //
//	red del proveedor:																				//
//																									//
//		APN: 				internet.itelcel.com 													//
//		Nombre de usuario: 	webgprs 																//
//		Contrasea: 		webgprs2002																//
//																									//
//	La aplicacion envia el siguiente mensaje para saber que se pueden enviar datos desde el servidor// 																								//
//																									//
//************************************************************************************************* //
//																									//
//                    Cliente TCP/IP QUECTEL UC15-BOLT												//
//																									//
// Control del relevador y lectura de la temperatura en el sistema BOLT								//
//																									//
// Para encender el relevador:          enciende relevador											//
// Para apagar el relevador:            apaga relevador												//
// Para leer la temperatura:            lee temperatura												//
// Para saber el estado de la puerta:   status puerta												//
//																									//
//	El cliente TCP/IP envia el estado de un switch al que esta conectado una						//
// 	puerta indicando si se ha abierto o cerrado														//
//																									//
//************************************************************************************************* //
//																									//
//	Escribe comando: 																				//
//																									//
//	Despues de este mesaje es posible enviar mensajes los que sern interpretados como comandos, 	//
//	si se reconoce el comando se ejecutar una accin en el sistema Bolt y este devolver un 		//
//	mensaje al servidor con la accin realizada, ejemplo:											//
//																									//
//	Servidor: 	enciende relevador																	//
//	Respuesta: 	relevador encendido!																//																		//
//																									//
//	Servidor: 	lee temperatura																		//
//	Respuesta:	Temp:  21.3 C  																	//
//																									//
//	Conexion entre en modulo UC15 y el sistema BOLT PIC18F2550									//
//																									//														
//	   |												 	   										//
//	   |											  												//
//	  _|__________				    	 ___________________			 ____________				//
//	 | 		      |	    	     	 	|		   	 		| J1		|			 |				//
//	 | 		      |	Serial	     	J3	| 	   				|-----------| 9 Volts DC |				//		
//	 |QUECTEL UC15|	 port	   	  		|     BOLT SYSTEM 	|  			|____________|				//
//	 |		   TXD|---------------------|RX      			|										//
//	 |		   RXD|---------------------|TX        			|										//
//	 |_________GND|---------------------|GND________________|										//
//		|									  ||||||												//
//		|							DB9		 _____________											//
//		|									|	LCD		  |											//
//		|									|_____________|											//
//	   _|____________																				//			
//	  |				 |																				//
//	  |	12 Volts DC  |																				//
//	  |______________|																				//
//																									//
//////////////////////////////////////////////////////////////////////////////////////////////////////
//																									//
//																									//
#define SERVER_IP   	"\"puntoflotante.ddns.net\"" // IP de la PC que corre el servidor			//
#define SERVER_PORT		"2020"		  	// Numero de puerto utilizado por el servidor				//
//																									//
//#define SERVER_IP    	"\"189.137.144.183\""  		 // IP de la PC que corre el servidor			//
//#define SERVER_PORT	"2022"			// Numero de puerto utilizado por el servidor				//	
//																									//
// Datos para acceder a la red TELCEL:																//
//																									//
#define APN			"\"internet.itelcel.com\""														//
#define User		"\"webgprs\""																	//
#define Password	"\"webgprs2002\""																//
//																									//
// Datos para acceder a la red Movistar:															//
//																									//
//#define APN		"\"internet.movistar.mx\""														//
//#define User		"\"movistar\""																	//
//#define Password	"\"movistar\""																	//
//																									//
//**************************************************************************************************//

#include <p18cxxx.h>
#include <delays.h>
#include <usart.h>
#include <string.h>
#include <stdio.h>		//libreria sprintf()
#include <stdlib.h>		//librerias atoi(),atof()
#include "18F2550BOLT.h"
#include "PUERTO-SERIAL-BOLT.h"
#include "ONEWIRE-BOLT.h"

#define CONV const far rom char *
#define INCOMING_SIZE 220		// Tamao del arreglo de datos recibidos
#define OK			0
#define ERROR		1
#define NO_ANSWER	2
#define puerta		SW1
#define SLEEP		while(1){}

// Prototipo de funciones
void rx_handler (void);											// Prototipo de ISR USART
void initConfiguration(void);									// Configuraciones iniciales
void writeLCDmsg(const far rom char * message);					// Escribe una cadena en el LCD
void commandProcess(void);										// Funcion que procesa los comandos
unsigned char sendATcommand(const far rom char * command, const far rom char * answer, unsigned int chars);
																// Envia los comandos AT al modulo QUECTEL UC15
void sendATdata(const far rom char *data);						// Envia datos al modulo QUECTEL UC15
unsigned char conect2SERVER(void);								// Funcion que conecta el modulo QUECTEL UC15 al servidor
void noConection(void);											// No es posible establecer la conexion

// Variables globales
volatile static char serialIncomingData[INCOMING_SIZE];			// Datos provenientes del puerto serie
volatile static unsigned char INDEX=0, INTERRUPT=0;				// Variables para el control de la interrupcion
volatile static unsigned char cont=0;
volatile static unsigned char puerta_actual, puerta_anterior=1;

extern void _startup( void ); 							// See c018i.c in your C18 compiler dir 
#pragma code _RESET_INTERRUPT_VECTOR = 0x000800 		// Vector de reset
void _reset( void ) 
{ 
    _asm goto _startup _endasm 
} 

#pragma code rx_interrupt = 0x000808					// Vector de interrupcion de alta prioridad
void rx_int (void)
{
	_asm goto rx_handler _endasm
}

#pragma code

#pragma interrupt rx_handler	// Funcion para el manejo de la interrupcion 
void rx_handler (void)
{
	serialIncomingData[INDEX++] = ReadUSART();	// Lee el buffer de entrada del puerto seriE del pic
	if(INDEX>INCOMING_SIZE) INDEX=0;
	PIR1bits.RCIF = 0;							// Limpia la bandera de interrupcion
	INTERRUPT=1;								// La interrupcion ha ocurrido
}
 
void main()						// Funcion principal
{
	unsigned char STATUS;		// Variable de control para monitorear el estado de la conexion
	TRISA=0X30;
	initConfiguration();		// Configuracines iniciales 
	TRISA=0X30;
	while(1){
		STATUS=conect2SERVER();		// Conecta el sistema BOLT-UC15 con un servidor remoto
		if(STATUS==OK){				// Si la conexion se ha establecido
			commandProcess();		// Procesa los comandos que envia el servidor
	}}	
} 

//
//	Funcion que envia todos los comandos AT necesarion para establecer la comunicacion con un 
//	servidor remoto a traves de GPRS en el modulo, se requiere de conocer la IP y el 
//	numero de puerto del servidor al que se requiere conectar. Ademas monitorea constantemente
//	el estado de la conexion, si no es posible establecerla debido a que alguno de los comandos
//	no puede procesarse la funcion retorna un valor de ERROR.
//
unsigned char conect2SERVER(void)
{
	unsigned char STATUS;
	if(!cont){
		writeLCDmsg((CONV)" Aplicacion de  ");
		delay_ms(1000);
		writeLCDmsg((CONV)"  telemetria y  ");
		delay_ms(1000);
		writeLCDmsg((CONV)" control remoto ");
		delay_ms(1000);
		writeLCDmsg((CONV)"del sistema BOLT");
		delay_ms(1000);
		writeLCDmsg((CONV)"y el modulo UC15");
		delay_ms(1000);
		writeLCDmsg((CONV)"  de QUECTEL    ");
		delay_ms(1000);
		writeLCDmsg((CONV)"servidor TCP-IP ");
		delay_ms(1000);
		writeLCDmsg((CONV)"  de internet   ");
		delay_ms(1000);
		writeLCDmsg((CONV)" usando la red  ");
		delay_ms(1000);
		writeLCDmsg((CONV)"de datos moviles");
		delay_ms(1000);
		writeLCDmsg((CONV)"BOLT-QUECTELUC15");
		delay_ms(1000);
	}cont++;
	goto INICIO;

CONECTA01:
	writeLCDmsg((CONV)"Conectar modulo!");
	delay_ms(5000);
	STATUS=sendATcommand((CONV)"AT",(CONV)"OK",3);
	if(STATUS!=OK) goto CONECTA01;
	writeLCDmsg((CONV)"Test moudule: OK");
	delay_ms(500);

REINICIA:
	writeLCDmsg((CONV)"REINICIANDO MODU");
	delay_ms(1000);
	writeLCDmsg((CONV)"EINICIANDO MODUL");
	delay_ms(1000);
	writeLCDmsg((CONV)"INICIANDO MODULO");
	delay_ms(1000);
	STATUS=sendATcommand((CONV)"AT+QPOWD=1",(CONV)"POWERED DOWN",115);
	delay_ms(8000);
	ClrWdt();
	if(STATUS!=OK) goto CONECTA01;
	else goto INICIO;

CONECTA:
	writeLCDmsg((CONV)"Conectar modulo!");
	ClrWdt();
	delay_ms(500);

INICIO:
	STATUS=sendATcommand((CONV)"AT",(CONV)"OK",3);
	if(STATUS!=OK) goto CONECTA;
	writeLCDmsg((CONV)"Test moudule: OK");
	delay_ms(500);

	writeLCDmsg((CONV)"Echo OFF:       ");
	STATUS=sendATcommand((CONV)"ATE0",(CONV)"OK",3);
	if(STATUS!=OK) goto CONECTA;
	writeLCDmsg((CONV)"Echo OFF: OK    ");
	delay_ms(500);

DEACT:
	writeLCDmsg((CONV)"Deact. contx:   ");
	STATUS=sendATcommand((CONV)"AT+QIDEACT=1",(CONV)"OK",3);
	if(STATUS!=OK) goto INICIO;
	writeLCDmsg((CONV)"Deact. contx: OK");
	delay_ms(500);

	writeLCDmsg((CONV)"Query SIM:      ");
	STATUS=sendATcommand((CONV)"AT+CPIN?",(CONV)"OK",20);
	if(STATUS!=OK) goto REINICIA;
	writeLCDmsg((CONV)"Query SIM: OK   ");
	delay_ms(500);

	writeLCDmsg((CONV)"CS service:     ");
	STATUS=sendATcommand((CONV)"AT+CREG?",(CONV)"OK",18);
	if(STATUS!=OK) goto REINICIA;
	writeLCDmsg((CONV)"CS service: OK  ");
	delay_ms(500);

	writeLCDmsg((CONV)"PS service:     ");
	STATUS=sendATcommand((CONV)"AT+CGREG?",(CONV)"OK",20);
	writeLCDmsg((CONV)"PS service: OK  ");
	delay_ms(500);

	writeLCDmsg((CONV)"Config. PDP:    ");
	STATUS=sendATcommand((CONV)"AT+QICSGP=1,1,"APN","User","Password",1",(CONV)"OK",3);
	writeLCDmsg((CONV)"Config. PDP: OK ");
	delay_ms(500);
	
	writeLCDmsg((CONV)"Activate PDP:   ");
	STATUS=sendATcommand((CONV)"AT+QIACT=1",(CONV)"OK",3);
	if(STATUS!=OK) goto DEACT;
	writeLCDmsg((CONV)"Activate PDP: OK");
	delay_ms(500);

	writeLCDmsg((CONV)"Query IP add:   ");
	STATUS=sendATcommand((CONV)"AT+QIACT?",(CONV)"OK",30);
	if(STATUS!=OK) goto DEACT;
	writeLCDmsg((CONV)"Query IP add: OK");
	delay_ms(500);
	
	writeLCDmsg((CONV)"OpeningConection");
	STATUS=sendATcommand((CONV)"AT+QIOPEN=1,0,\"TCP\","SERVER_IP","SERVER_PORT",0,1",(CONV)"0,0",20);
	if(STATUS!=OK) goto DEACT;
	writeLCDmsg((CONV)"   CONNECTED    ");
	delay_ms(500);
	goto SEND_MESSAGE;

SEND_MESSAGE:
	STATUS=sendATcommand((CONV)"AT+QISEND=0",(CONV)">",1);
	sendATdata((CONV)"*************************************************************************");
	sendATdata((CONV)"");
	sendATdata((CONV)"                    Cliente TCP/IP QUECTEL UC15 - BOLT");
	sendATdata((CONV)"");
	sendATdata((CONV)" Control del relevador y lectura de la temperatura en el sistema BOLT");
	sendATdata((CONV)"");
	sendATdata((CONV)" Para encender el relevador:          enciende relevador");
	sendATdata((CONV)" Para apagar el relevador: 			apaga relevador");
	sendATdata((CONV)" Para leer la temperatura: 			lee temperatura");
	sendATdata((CONV)" Para saber el estado de la puerta: 	status puerta");
	sendATdata((CONV)"");
	sendATdata((CONV)" El cliente TCP/IP envia el estado de un switch al que esta conectado una");
	sendATdata((CONV)" puerta indicando si se ha abierto o cerrado");
	sendATdata((CONV)"");
	sendATdata((CONV)"***************************************************************************");
	sendATdata((CONV)"");
	sendATdata((CONV)" Escribe comando:"); 
	while(BusyUSART());
	putcUSART((char)26);
	delay_ms(5000);
	return OK;		
}

//	
//	Funcion que envia un comando AT al modulo QUECTEL UC15, recibe el comando AT, la respuesta esperada, el retardo para el
//	procesamiento del comando AT por parte del modulo QUECTEL UC15, y un tiempo limite, si el modulo QUECTEL UC15 no responde en 
//	el lapso de tiempo indicado por este numero la funcion retorna el valor NO_ANSWER
//
unsigned char sendATcommand(const far rom char * command, const far rom char * answer, unsigned int chars)
{
	unsigned long int time;
	int i;
	INTERRUPT=0;
	for(i=0; i<strlenpgm(command); i++) 
	{
		while(BusyUSART());		// Espera a que este disponible el modulo USART
		putcUSART(command[i]);	// envia cada uno de los caracteres del comando AT
	}

	while(BusyUSART());
	putcUSART(13);
	ClrWdt();

	for(time=0;time<(8000000);time++)
	{
		if(INDEX>chars)
		{
			delay_ms(1000);

			if(strstrrampgm(serialIncomingData,answer)) 
			{	
				INDEX=0;
				INTERRUPT=0;
				for(i=0;i<INCOMING_SIZE;i++) serialIncomingData[i]=0;
				return OK;
			}
			else 
			{	
				INDEX=0;
				INTERRUPT=0;
				for(i=0;i<INCOMING_SIZE;i++) serialIncomingData[i]=0;
				return ERROR;
			}
		}
		else continue;
	}
	INDEX=0;
	INTERRUPT=0;
	for(i=0;i<INCOMING_SIZE;i++) serialIncomingData[i]=0;
	return NO_ANSWER;
}

void initConfiguration(void)	// Configuraciones iniciales para el sistema Bolt, cofigura el LCD, los puertos, el relevador
{								// y el puerto serie en hardware
	int initIndex;			// Variable local para limpiar el arreglo que contendra los datos recibidos
	init_bolt();			
	InitLCD();
	activa_serial();		
	RCONbits.IPEN = 1;		// Habilita prioridad en las interrupciones
	IPR1bits.RCIP = 1;		// Alta prioridad para la interrupcion por llegada de datos
	INTCONbits.GIEH = 1;	// Habilita todas las interrupciones de alta prioridad
	for(initIndex=0;initIndex<INCOMING_SIZE;initIndex++) serialIncomingData[initIndex]=0;
	WDTCONbits.SWDTEN = 1; 	// Habilita el temporizador del perro guardian
}

void sendATdata(const far rom char *data)	// Envia los datos al modulo QUECTEL UC15 
{
	int i;
	for(i=0; i<strlenpgm(data); i++) 
	{
		while(BusyUSART());				// Espera a que el puerto serie este desocupado
		putcUSART(data[i]);				// Escribe cada uno de los caracteres al puerto serie
	}
	while(BusyUSART());
	putcUSART('\r');					// Retorno de carro
	while(BusyUSART());
	putcUSART((char)10);
}

void writeLCDmsg(const far rom char * message)		// Escribe la cadena "message" al LCD
{
	unsigned char i;
	ClearScreen();							// Limpia la pantalla
	GotoXY(0,0);							// Ve a la posicion (0,0)
	for(i=0; i<8; i++) PutLCD(message[i]);	// Imprime cada uno de los caracteres de la cadena del argumento
	GotoXY(1,0);
	for(i=8; i<strlenpgm(message);i++) PutLCD(message[i]);
}

//
//	Funcion que procesa los mensajes provenientes del servidor, si es un comando reconocido devuelve un mensaje 
//	indicando el efecto del comando.
//
void commandProcess(void)		
{
	int k;
	unsigned char con=0;
	float temperatura;
	int entero, decimal;
	unsigned char STATUS;				// 1 cerrado
	//unsigned char puerta_anterior=1;	// 0 abierto
	unsigned long j;
	char on[]=		"enciende relevador";	// Cadenas de caracteres para el reconocimiento de comandodos
	char off[]=		"apaga relevador";		
	char tempe[]=	"lee temperatura";
	char STpuerta[]="status puerta";
	char closed[]=  "closed";
	char sendOK[]=  "SEND OK";
	char temp[20];
	INDEX=0; 
	INTERRUPT=0;
	WDTCONbits.SWDTEN = 1; // habilita el perro guardian
	while(1){
	for(j=0;j<10000000;j++){
		if(INTERRUPT)						// Si la interrupcion ha ocurrido se procesan los datos.		
		{
			delay_ms(1000);
			if(strstr(serialIncomingData,closed))
			{
				writeLCDmsg((CONV)"Servidor offline");
				delay_ms(8000);
				writeLCDmsg((CONV)"  Reiniciando!  ");
				goto ESCAPA;
			}
			else if(strstr(serialIncomingData,sendOK)){
				con++;
			}
			else if(strstr(serialIncomingData,on)) 
			{
				RELE=1;
				writeLCDmsg((CONV)"  Relevador ON  ");
				STATUS=sendATcommand((CONV)"AT+QISEND=0",(CONV)">",1);
				if(STATUS==OK)
				{
					sendATdata((CONV)"Relevador encendido");
					while(BusyUSART());
					putcUSART((char)26);
				}
			}
			else if(strstr(serialIncomingData,off)) 
			{
				RELE=0;
				writeLCDmsg((CONV)" Relevador OFF  ");
				STATUS=sendATcommand((CONV)"AT+QISEND=0",(CONV)">",1);
				if(STATUS==OK)
				{
					sendATdata((CONV)"Relevador apagado");
					while(BusyUSART());
					putcUSART((char)26);
				}
			}
			else if(strstr(serialIncomingData,tempe))
			{
				temperatura=ds1820_read();
				entero=temperatura;						// Parte entera
				decimal=(temperatura-entero)*10;		// Parte decimal
				sprintf(temp,(CONV)"Temp: %3d.%d C  \n\r",entero,decimal);		// Convierte a una cadena de caracteres
				ClearScreen();
				GotoXY(0,0);
				for(k=0; k<8; k++) PutLCD(temp[k]);
				GotoXY(1,0);
				for(k=8; k<11;k++) PutLCD(temp[k]);
				PutLCD(32);							//espacio
				PutLCD(214);						//simbolo de grados
				PutLCD('C');
				PutLCD(32);							//espacio
				PutLCD(32);							//espacio
				STATUS=sendATcommand((CONV)"AT+QISEND=0",(CONV)">",1);
				if(STATUS==OK)
				{
					for(k=0; k<strlen(temp); k++) 
					{
						while(BusyUSART());
						putcUSART(temp[k]);
					}
					while(BusyUSART());
					putcUSART('\r');
					delay_ms(200);
					while(BusyUSART());
					putcUSART((char)26);
				}
			}
			else if(strstr(serialIncomingData,STpuerta))
			{
				TRISA=0X30;
				puerta_actual=puerta;
				if(puerta_actual==0)
				{
					writeLCDmsg((CONV)" Puerta abierta ");
					STATUS=sendATcommand((CONV)"AT+QISEND=0",(CONV)">",1);
					sendATdata((CONV)"Puerta abierta");
					while(BusyUSART());
					putcUSART((char)26);
				}
				else if(puerta_actual==1)
				{
					writeLCDmsg((CONV)" Puerta cerrada ");
					STATUS=sendATcommand((CONV)"AT+QISEND=0",(CONV)">",1);
					sendATdata((CONV)"Puerta cerrada");
					while(BusyUSART());
					putcUSART((char)26);
				}				
			}
			else
			{
				STATUS=sendATcommand((CONV)"AT+QISEND=0",(CONV)">",1);		// Si el comando no se reconoce
				if(STATUS==OK)												// se envia el mensasaje 
				{															// "No se reconoce comando"
					sendATdata((CONV)"No se reconoce comando");
					while(BusyUSART());
					putcUSART((char)26);
				}
				writeLCDmsg((CONV)"Ninguna accion  ");
			}
			INDEX=0;
			INTERRUPT=0;
			for(k=0;k<INCOMING_SIZE;k++) serialIncomingData[k]=0;
	
		}ClrWdt();
		TRISA=0X30;
		puerta_actual=puerta;
		if(puerta_actual!=puerta_anterior)
		{
			if(puerta_actual==0)
			{
				writeLCDmsg((CONV)"Puerta se ha ab.");
				STATUS=sendATcommand((CONV)"AT+QISEND=0",(CONV)">",1);
				sendATdata((CONV)"La puerta se ha abierto!");
				while(BusyUSART());
				putcUSART((char)26);
			}
			else if(puerta_actual==1)
			{
				writeLCDmsg((CONV)"Puerta se ha ce.");
				STATUS=sendATcommand((CONV)"AT+QISEND=0",(CONV)">",1);
				sendATdata((CONV)"La puerta de ha cerrado!");
				while(BusyUSART());
				putcUSART((char)26);
			}
			delay_ms(500);
			puerta_anterior=puerta_actual;
		}}	
		STATUS=sendATcommand((CONV)"AT+QISEND=0",(CONV)">",1);
		sendATdata((CONV)"...");
		while(BusyUSART());
		putcUSART((char)26);
	}
ESCAPA: con++;
}
